const path = require('path')
const postcssNormalize = require('postcss-normalize')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
const appSrc = path.resolve(__dirname, '.')

const postCssLoader = {
  // Options for PostCSS as we reference these options twice
  // Adds vendor prefixing based on your specified browser support in
  // package.json
  loader: require.resolve('postcss-loader'),
  options: {
    // Necessary for external CSS imports to work
    // https://github.com/facebook/create-react-app/issues/2677
    ident: 'postcss',
    plugins: () => [
      require('postcss-flexbugs-fixes'),
      require('postcss-preset-env')({
        autoprefixer: {
          flexbox: 'no-2009'
        },
        stage: 3
      }),
      // Adds PostCSS Normalize as the reset css with default options,
      // so that it honors browserslist config in package.json
      // which in turn let's users customize the target behavior as per their needs.
      postcssNormalize()
    ],
    sourceMap: false
  }
}


module.exports = {
  mode: 'production',
  entry: './visualization.tsx',
  output: {
    filename: 'visualization.js',
    path: path.resolve(appSrc, "../"),
    libraryTarget: 'amd'
  },
  resolve: {
    modules: ['node_modules', appSrc],
    extensions: ['.ts', '.tsx', '.js', '.jsx']
  },
  externals: {
    react: 'react',
    'react-dom': 'react-dom',
    "@qn-pandora/visualization-sdk": "@qn-pandora/visualization-sdk"
  },
  module: {
    rules: [
      {
        test: /\.tsx?$/,
        exclude: /node_modules/,
        use: [
          {
            loader: 'babel-loader',
            options: {
              presets: ['@babel/preset-env'],
              plugins: [['@babel/plugin-proposal-decorators', { legacy: true }]]
            }
          },
          'ts-loader'
        ]
      },
      {
        test: /\.js$/,
        exclude: /node_modules/,
        use: [
          {
            loader: 'babel-loader',
            options: {
              presets: ['@babel/preset-env'],
              plugins: [['@babel/plugin-proposal-decorators', { legacy: true }]]
            }
          }
        ]
      },
      {
        test: /\.m?less$/,
        use: [
          {
            loader: 'style-loader'
          },
          MiniCssExtractPlugin.loader,
          {
            loader: 'css-loader',
            options: {
              importLoaders: 2,
              sourceMap: false,
              modules: true,
              localsConvention: 'camelCaseOnly'
            }
          },
          postCssLoader,
          { loader: 'less-loader', options: { javascriptEnabled: true } }
        ],
        sideEffects: true
      },
      {
        test: /\.css$/,
        use: [
          {
            loader: 'style-loader'
          },
          MiniCssExtractPlugin.loader,
          {
            loader: 'css-loader',
            options: {
              importLoaders: 1,
              sourceMap: false
            }
          },
          postCssLoader
        ]
      }
    ]
  },
  plugins: [
    new MiniCssExtractPlugin({
      filename: 'visualization.css'
    })
  ]
}
